@@ -3,10 +3,11 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include <assert.h>
 #include <errno.h>
 #include <stddef.h>
-#include <stdio.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -298,6 +299,10 @@ chunks_directory_write(const char * cachepath, RWHASHTAB * HT,
 	char * s;
 	int fd;
 
+	/* The caller must pass the cachepath, and a suffix to use. */
+	assert(cachepath != NULL);
+	assert(suff != NULL);
+
 	/* Construct the path to the new chunk directory. */
 	if (asprintf(&s, "%s/directory%s", cachepath, suff) == -1) {
 		warnp("asprintf");
@@ -341,6 +346,9 @@ chunks_directory_write(const char * cachepath, RWHASHTAB * HT,
 		goto err1;
 	}
 
+	/* Free string allocated by asprintf. */
+	free(s);
+
 	/* Success! */
 	return (0);
 
@@ -386,6 +394,11 @@ chunks_directory_commit(const char * cachepath, const char * osuff,
 	char * s;
 	char * t;
 
+	/* The caller must pass the cachepath, and suffices to use. */
+	assert(cachepath != NULL);
+	assert(nsuff != NULL);
+	assert(osuff != NULL);
+
 	/* Construct file names. */
 	if (asprintf(&s, "%s/directory%s", cachepath, nsuff) == -1) {
 		warnp("asprintf");
