@@ -78,6 +78,7 @@ static struct option {
 	{ "confirmation",         0, 'w' },
 	{ "create",               0, 'c' },
 	{ "creationtime",         1, OPTION_CREATIONTIME },
+	{ "csv-file",		  1, OPTION_CSV_FILE },
 	{ "dereference",	  0, 'L' },
 	{ "directory",            1, 'C' },
 	{ "disk-pause",		  1, OPTION_DISK_PAUSE },
@@ -96,6 +97,7 @@ static struct option {
 	{ "insane-filesystems",	  0, OPTION_INSANE_FILESYSTEMS },
 	{ "insecure",             0, 'P' },
 	{ "interactive",          0, 'w' },
+	{ "keep-going",           0, OPTION_KEEP_GOING },
 	{ "keep-newer-files",     0, OPTION_KEEP_NEWER_FILES },
 	{ "keep-old-files",       0, 'k' },
 	{ "keyfile",		  1, OPTION_KEYFILE },
@@ -156,6 +158,7 @@ static struct option {
 	{ "totals",		  0, OPTION_TOTALS },
 	{ "unlink",		  0, 'U' },
 	{ "unlink-first",	  0, 'U' },
+	{ "verify-config",	  0, OPTION_VERIFY_CONFIG },
 	{ "verbose",              0, 'v' },
 	{ "version",              0, OPTION_VERSION },
 	{ "verylowmem",		  0, OPTION_VERYLOWMEM },
