@@ -29,10 +29,18 @@ tarsnap_mode_d(struct bsdtar *bsdtar)
 		if (bsdtar->verbose && (bsdtar->ntapes > 1))
 			fprintf(stderr, "Deleting archive \"%s\"\n",
 			    bsdtar->tapenames[i]);
-		if (deletetape(d, bsdtar->machinenum, bsdtar->cachedir,
+		switch (deletetape(d, bsdtar->machinenum, bsdtar->cachedir,
 		    bsdtar->tapenames[i], bsdtar->option_print_stats,
-		    bsdtar->ntapes > 1 ? 1 : 0))
-			goto err1;
+		    bsdtar->ntapes > 1 ? 1 : 0, bsdtar->option_csv_filename)) {
+		case 0:
+			break;
+		case 1:
+			if (bsdtar->option_keep_going)
+				break;
+			/* FALLTHROUGH */
+		default:
+			goto err2;
+		}
 	}
 
 	/* We've finished deleting archives. */
@@ -41,10 +49,13 @@ tarsnap_mode_d(struct bsdtar *bsdtar)
 	/* Success! */
 	return;
 
+err2:
+	deletetape_free(d);
 err1:
 	/* Failure! */
 	bsdtar_warnc(bsdtar, 0, "Error deleting archive");
-	exit(1);
+	bsdtar->return_value = 1;
+	return;
 }
 
 /*
@@ -90,11 +101,11 @@ tarsnap_mode_r(struct bsdtar *bsdtar)
 
 err2:
 	readtape_close(d);
-
 err1:
 	/* Failure! */
 	bsdtar_warnc(bsdtar, 0, "Error reading archive");
-	exit(1);
+	bsdtar->return_value = 1;
+	return;
 }
 
 /*
@@ -112,7 +123,7 @@ tarsnap_mode_print_stats(struct bsdtar *bsdtar)
 		goto err1;
 
 	/* Print statistics about the archive set. */
-	if (statstape_printglobal(d))
+	if (statstape_printglobal(d, bsdtar->option_csv_filename))
 		goto err2;
 
 	if (bsdtar->ntapes == 0) {
@@ -120,13 +131,22 @@ tarsnap_mode_print_stats(struct bsdtar *bsdtar)
 	} else if ((bsdtar->tapenames[0][0] == '*') &&
 	    (bsdtar->tapenames[0][1] == '\0')) {
 		/* User wants statistics on all archives. */
-		if (statstape_printall(d))
+		if (statstape_printall(d, bsdtar->option_csv_filename))
 			goto err2;
 	} else {
 		/* User wants statistics about specific archive(s). */
 		for (i = 0; i < bsdtar->ntapes; i++) {
-			if (statstape_print(d, bsdtar->tapenames[i]))
+			switch (statstape_print(d, bsdtar->tapenames[i],
+			    bsdtar->option_csv_filename)) {
+			case 0:
+				break;
+			case 1:
+				if (bsdtar->option_keep_going)
+					break;
+				/* FALLTHROUGH */
+			default:
 				goto err2;
+			}
 		}
 	}
 
@@ -142,7 +162,8 @@ err2:
 err1:
 	/* Failure! */
 	bsdtar_warnc(bsdtar, 0, "Error generating archive statistics");
-	exit(1);
+	bsdtar->return_value = 1;
+	return;
 }
 
 /*
@@ -173,7 +194,8 @@ err2:
 err1:
 	/* Failure! */
 	bsdtar_warnc(bsdtar, 0, "Error listing archives");
-	exit(1);
+	bsdtar->return_value = 1;
+	return;
 }
 
 /*
@@ -185,7 +207,7 @@ tarsnap_mode_fsck(struct bsdtar *bsdtar, int prune, int whichkey)
 
 	if (fscktape(bsdtar->machinenum, bsdtar->cachedir, prune, whichkey)) {
 		bsdtar_warnc(bsdtar, 0, "Error fscking archives");
-		exit(1);
+		goto err0;
 	}
 
 	/*
@@ -197,11 +219,16 @@ tarsnap_mode_fsck(struct bsdtar *bsdtar, int prune, int whichkey)
 	 */
 	if (ccache_remove(bsdtar->cachedir)) {
 		bsdtar_warnc(bsdtar, 0, "Error removing chunkification cache");
-		exit(1);
+		goto err0;
 	}
 
 	/* Success! */
 	return;
+
+err0:
+	/* Failure! */
+	bsdtar->return_value = 1;
+	return;
 }
 
 /*
@@ -217,23 +244,25 @@ tarsnap_mode_nuke(struct bsdtar *bsdtar)
 	if (fgets(s, 100, stdin) == NULL) {
 		bsdtar_warnc(bsdtar, 0,
 		    "Error reading string from standard input");
-		exit(1);
+		goto err0;
 	}
 	if (strcmp(s, "No Tomorrow\n")) {
 		bsdtar_warnc(bsdtar, 0, "You didn't type 'No Tomorrow'");
-		exit(1);
+		goto err0;
 	}
 
-	if (nuketape(bsdtar->machinenum))
-		goto err1;
+	if (nuketape(bsdtar->machinenum)) {
+		bsdtar_warnc(bsdtar, 0, "Error nuking archives");
+		goto err0;
+	}
 
 	/* Success! */
 	return;
 
-err1:
+err0:
 	/* Failure! */
-	bsdtar_warnc(bsdtar, 0, "Error nuking archives");
-	exit(1);
+	bsdtar->return_value = 1;
+	return;
 }
 
 /*
@@ -252,5 +281,6 @@ tarsnap_mode_recover(struct bsdtar *bsdtar, int whichkey)
 err1:
 	/* Failure! */
 	bsdtar_warnc(bsdtar, 0, "Error recovering archive");
-	exit(1);
+	bsdtar->return_value = 1;
+	return;
 }
