@@ -60,22 +60,33 @@ err0:
 }
 
 /**
- * deletetape(d, machinenum, cachedir, tapename, printstats, withname):
+ * deletetape(d, machinenum, cachedir, tapename, printstats, withname,
+ *     csv_filename):
  * Delete the specified tape, and print statistics to stderr if requested.
  * If ${withname} is non-zero, print statistics with the archive name, not
- * just as "This archive".
+ * just as "This archive".  Return 0 on success, 1 if the tape does not exist,
+ * or -1 on other errors.  If ${csv_filename} is specified, output in CSV
+ * format instead of to stderr.
  */
 int
 deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
-    const char * tapename, int printstats, int withname)
+    const char * tapename, int printstats, int withname,
+    const char * csv_filename)
 {
 	struct tapemetadata tmd;
 	CHUNKS_D * C;		/* Chunk layer delete cookie. */
 	STORAGE_D * S;		/* Storage layer delete cookie. */
-	STORAGE_R * SR = d->S;		/* Storage layer read cookie. */
+	STORAGE_R * SR = d->S;	/* Storage layer read cookie. */
 	int lockfd;
 	uint8_t lastseq[32];
 	uint8_t seqnum[32];
+	int rc = -1;		/* Presume error was not !found. */
+	FILE * output = stderr;
+	int csv = 0;
+
+	/* Should we output to a CSV file? */
+	if (csv_filename != NULL)
+		csv = 1;
 
 	/* Lock the cache directory. */
 	if ((lockfd = multitape_lock(cachedir)) == -1)
@@ -99,20 +110,27 @@ deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
 	storage_read_cache_limit(SR, 100 * chunks_delete_getdirsz(C));
 
 	/* Read archive metadata. */
-	if (multitape_metadata_get_byname(SR, NULL, &tmd, tapename, 0))
+	switch (multitape_metadata_get_byname(SR, NULL, &tmd, tapename, 0)) {
+	case 0:
+		break;
+	case 1:
+		rc = 1;
+		/* FALLTHROUGH */
+	default:
 		goto err3;
+	}
 
 	/* Delete chunks. */
 	if (multitape_chunkiter_tmd(SR, NULL, &tmd, callback_delete, C, 0))
-		goto err4;
+		goto err5;
 
 	/* Delete archive index. */
 	if (multitape_metaindex_delete(S, C, &tmd))
-		goto err4;
+		goto err5;
 
 	/* Delete archive metadata. */
 	if (multitape_metadata_delete(S, C, &tmd))
-		goto err4;
+		goto err5;
 
 	/* Free tape metadata. */
 	multitape_metadata_free(&tmd);
@@ -122,9 +140,18 @@ deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
 		goto err3;
 
 	/* Print statistics if they were requested. */
-	if ((printstats != 0) &&
-	    chunks_delete_printstats(stderr, C, withname ? tapename : NULL))
-		goto err3;
+	if (printstats != 0) {
+		if (csv && (output = fopen(csv_filename, "wt")) == NULL)
+			goto err3;
+
+		/* Actually print statistics. */
+	    	if (chunks_delete_printstats(output, C,
+		    withname ? tapename : NULL, csv))
+			goto err4;
+
+		if (csv && fclose(output))
+			goto err3;
+	}
 
 	/* Close storage and chunk layer cookies. */
 	if (chunks_delete_end(C))
@@ -142,8 +169,11 @@ deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
 	/* Success! */
 	return (0);
 
-err4:
+err5:
 	multitape_metadata_free(&tmd);
+err4:
+	if (output != stderr)
+		fclose(output);
 err3:
 	chunks_delete_free(C);
 err2:
@@ -152,7 +182,7 @@ err1:
 	close(lockfd);
 err0:
 	/* Failure! */
-	return (-1);
+	return (rc);
 }
 
 /**
@@ -163,6 +193,10 @@ void
 deletetape_free(TAPE_D * d)
 {
 
+	/* Behave consistently with free(NULL). */
+	if (d == NULL)
+		return;
+
 	/* Close the storage layer read cookie. */
 	storage_read_free(d->S);
 
