@@ -8,6 +8,7 @@
 #include "crypto.h"
 #include "dirutil.h"
 #include "hexify.h"
+#include "imalloc.h"
 #include "multitape_internal.h"
 #include "storage.h"
 #include "sysendian.h"
@@ -165,7 +166,7 @@ phase1(uint64_t machinenum, STORAGE_D * SD, STORAGE_R * SR,
 		goto err0;
 
 	/* Allocate space for nfiles tapemetadata structures. */
-	if ((mdats = malloc(nfiles * sizeof(struct tapemetadata *))) == NULL)
+	if (IMALLOC(mdats, nfiles, struct tapemetadata *))
 		goto err1;
 	nvalids = 0;
 
