@@ -3,6 +3,7 @@
 #include <sys/file.h>
 #include <sys/stat.h>
 
+#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <inttypes.h>
@@ -63,6 +64,9 @@ multitape_docheckpoint(const char * cachedir, uint64_t machinenum,
 	uint8_t ckptnonce[32];
 	uint8_t seqnum_ckptnonce[64];
 
+	/* We need a cachedir. */
+	assert(cachedir != NULL);
+
 	/* Make sure ${cachedir} is flushed to disk. */
 	if (dirutil_fsyncdir(cachedir))
 		goto err0;
@@ -155,6 +159,9 @@ multitape_checkpoint(const char * cachedir, uint64_t machinenum,
 	uint8_t ckptnonce[32];
 	uint8_t seqnum_ckptnonce[64];
 
+	/* We need a cachedir. */
+	assert(cachedir != NULL);
+
 	/* Generate random checkpoint nonce. */
 	if (crypto_entropy_read(ckptnonce, 32))
 		goto err0;
@@ -197,6 +204,9 @@ multitape_docommit(const char * cachedir, uint64_t machinenum, uint8_t key)
 	char * s, * t;
 	uint8_t seqnum[32];
 
+	/* We need a cachedir. */
+	assert(cachedir != NULL);
+
 	/* Make sure ${cachedir} is flushed to disk. */
 	if (dirutil_fsyncdir(cachedir))
 		goto err0;
@@ -283,6 +293,9 @@ multitape_commit(const char * cachedir, uint64_t machinenum,
 {
 	char * s;
 
+	/* We need a cachedir. */
+	assert(cachedir != NULL);
+
 	/* Make ${cachedir}/commit_m point to ${seqnum}. */
 	if (asprintf(&s, "%s/commit_m", cachedir) == -1) {
 		warnp("asprintf");
@@ -317,6 +330,9 @@ multitape_lock(const char * cachedir)
 	char * s;
 	int fd;
 
+	/* We need a cachedir. */
+	assert(cachedir != NULL);
+
 	/* Open ${cachedir}/lockf. */
 	if (asprintf(&s, "%s/lockf", cachedir) == -1) {
 		warnp("asprintf");
@@ -384,6 +400,9 @@ multitape_sequence(const char * cachedir, uint8_t seqnum[32])
 {
 	char * s;
 
+	/* We need a cachedir. */
+	assert(cachedir != NULL);
+
 	/* Read the link ${cachedir}/cseq. */
 	if (asprintf(&s, "%s/cseq", cachedir) == -1) {
 		warnp("asprintf");
